@@ -1,5 +1,6 @@
 import sys
 from typing import Awaitable, Callable, Optional, TYPE_CHECKING
+from rich.segment import Segment
 
 if sys.version_info >= (3, 8):
     from typing import Protocol
@@ -26,3 +27,5 @@ class EventTarget(Protocol):
 
 
 MessageHandler = Callable[["Message"], Awaitable]
+
+Lines = list[list[Segment]]
\ No newline at end of file
