@@ -25,13 +25,8 @@ class Event(Message):
 
 
 class Null(Event):
-    def can_batch(self, event: Event) -> bool:
-        return isinstance(event, Null)
-
-
-class Repaint(Event):
-    def can_batch(self, event: Event) -> bool:
-        return isinstance(event, Repaint)
+    def can_batch(self, message: Message) -> bool:
+        return isinstance(message, Null)
 
 
 class ShutdownRequest(Event):
@@ -62,7 +57,7 @@ class Idle(Event):
     """Sent when there are no more items in the message queue."""
 
 
-class Action(Event):
+class Action(Event, bubble=True):
     __slots__ = ["action"]
 
     def __init__(self, sender: MessageTarget, action: str) -> None:
@@ -96,6 +91,14 @@ class Unmount(Event):
     pass
 
 
+class Show(Event):
+    """Widget has become visible."""
+
+
+class Hide(Event):
+    """Widget has been hidden."""
+
+
 class InputEvent(Event, bubble=True):
     pass
 
@@ -121,6 +124,8 @@ class MouseEvent(InputEvent):
         sender: MessageTarget,
         x: int,
         y: int,
+        delta_x: int,
+        delta_y: int,
         button: int,
         shift: bool,
         meta: bool,
@@ -131,6 +136,8 @@ class MouseEvent(InputEvent):
         super().__init__(sender)
         self.x = x
         self.y = y
+        self.delta_x = delta_x
+        self.delta_y = delta_y
         self.button = button
         self.shift = shift
         self.meta = meta
@@ -141,6 +148,8 @@ class MouseEvent(InputEvent):
     def __rich_repr__(self) -> RichReprResult:
         yield "x", self.x
         yield "y", self.y
+        yield "delta_x", self.delta_x, 0
+        yield "delta_y", self.delta_y, 0
         if self.screen_x != self.x:
             yield "screen_x", self.screen_x
         if self.screen_y != self.y:
@@ -155,6 +164,8 @@ class MouseEvent(InputEvent):
             self.sender,
             x=self.x + x,
             y=self.y + y,
+            delta_x=self.delta_x,
+            delta_y=self.delta_y,
             button=self.button,
             shift=self.shift,
             meta=self.meta,
